(document:surround "/std/frame")
(document:insert "/std/functions")
(document:insert "/standalone/profile")

;;; Functions
(define *step* (make-cell 0))
(define *step-list* (make-cell '()))

(define (step) (cell-ref *step*))
(define (++step) (cell-set! *step* (+ (step) 1)) (step))
(define (--step) (cell-set! *step* (- (step) 1)) (step))
(define (last) (- (length (cell-ref *step-list*)) 1))

(define (url) (car (list-ref (cell-ref *step-list*) (step))))
(define (topic) (cdr (list-ref (cell-ref *step-list*) (step))))

;;register globals
(set-global! 'frame:next (make-cell #f))
(set-global! 'frame:back (make-cell #f))
;;get globals to avoid to be context specific
(define next-cb (global 'frame:next))
(define back-cb (global 'frame:back))

(define (call-cell cell)
  (let ((proc (cell-ref cell)))
    (and (procedure? proc) (proc))))

(define (popup-overflow proc cb)
  (document:popup-critical
   (string-append "Please remove " proc " call from " cb " function"))
   'cancel)

(define (sure-yes key name list)
  (or (member key list)
      (wizard-id action-add (vector key name))))

(define (sure-no key name list)
  (and (member key list)
       (wizard-id action-remove key)))

(define (action-name x) (vector-ref x 0))

(define (check-buttons step last)
  (with-translation _ "alterator-wizard"
                    (let ((buttons (map action-name (wizard-id actions))))
                      (cond
                       ((= step 0)
                        (sure-yes 'forward (_ "Next") buttons)
                        (sure-no 'backward (_ "Previous") buttons)
                        (sure-no 'finish (_ "Finish") buttons))
                       ((< 0 step last)
                        (sure-yes 'forward (_ "Next") buttons)
                        (sure-yes 'backward (_ "Previous") buttons)
                        (sure-no 'finish (_ "Finish") buttons))
                       ((= step last)
                        (sure-no 'forward (_ "Next") buttons)
                        (sure-yes 'backward (_ "Previous") buttons)
                        (sure-yes 'finish (_ "Finish") buttons))))
                    #t))

(define (frame:replace+help url . args)
  (apply frame:replace url args)
  (frame:help))

(define (go-next)
  ;;protect from stack overflow
  (frame:set-next (thunk (popup-overflow "frame:next" "frame:on-next")))
  (let ((last (last)))
    (and (< (step) last)
         (not (eq? (call-cell next-cb) 'cancel))
         (wizard-id current-step (++step))
         (check-buttons (step) last)
         (frame:replace+help (url) 'frame:direction 'next)))
  (frame:set-next go-next))

(define (go-back)
  ;;protect from stack overflow
  (frame:set-back (thunk (popup-overflow "frame:back" "frame:on-back")))
  (let ((last (last)))
    (and (> (step) 0)
         (not (eq? (call-cell back-cb) 'cancel))
         (wizard-id current-step (--step))
         (check-buttons (step) last)
         (frame:replace+help (url) 'frame:direction 'back)))
  (frame:set-back go-back))

(define (go-end)
  (frame:set-end (thunk (popup-overflow "frame:end" "frame:on-next")))
  (or (eq? 'cancel (call-cell next-cb))
      (document:end)))

(define (name+help x)
  (cons (woo-get-option x 'name)
        (woo-get-option x 'help)))

(define (label+icon x)
  (cons (woo-get-option x 'label)
        (woo-get-option x 'icon)))

;;; UI
(frame:set-next go-next)
(frame:set-back go-back)
(frame:set-end go-end)

(frame:make-wizardface
 title "Installer"
 (frame:make-page))

;;; Logic
(document:root
 (when loaded
   (let ((step-list (woo-list "/step-list")))

     (cell-set! *step-list* (map name+help step-list))
     (wizard-id steps (map label+icon step-list)
                current-step 0)
     (apply-actions wizard-id
                    'finish frame:end
                    'forward frame:next
                    'backward frame:back)
     (frame:replace+help (url) 'frame:direction 'next)
     (check-buttons (step) (last))
     (page-id loaded))))
